import { ContextService, RenderingPluginContribution } from '@antv/g';

var NAMED_TAG = "named";
var INJECT_TAG = "inject";
var TAGGED = "inversify:tagged";
var TAGGED_PROP = "inversify:tagged_props";
var PARAM_TYPES = "inversify:paramtypes";
var DESIGN_PARAM_TYPES = "design:paramtypes";

var idCounter = 0;
function id() {
    return idCounter++;
}

var DUPLICATED_INJECTABLE_DECORATOR = "Cannot apply @injectable decorator multiple times.";
var DUPLICATED_METADATA = "Metadata key was used more than once in a parameter:";
var UNDEFINED_INJECT_ANNOTATION = function (name) {
    return "@inject called with undefined this could mean that the class " + name + " has " +
        "a circular dependency problem. You can use a LazyServiceIdentifer to  " +
        "overcome this limitation.";
};
var INVALID_DECORATOR_OPERATION = "The @inject @multiInject @tagged and @named decorators " +
    "must be applied to the parameters of a class constructor or a class property.";

var Metadata = (function () {
    function Metadata(key, value) {
        this.key = key;
        this.value = value;
    }
    Metadata.prototype.toString = function () {
        if (this.key === NAMED_TAG) {
            return "named: " + this.value.toString() + " ";
        }
        else {
            return "tagged: { key:" + this.key.toString() + ", value: " + this.value + " }";
        }
    };
    return Metadata;
}());

function tagParameter(annotationTarget, propertyName, parameterIndex, metadata) {
    var metadataKey = TAGGED;
    _tagParameterOrProperty(metadataKey, annotationTarget, propertyName, metadata, parameterIndex);
}
function tagProperty(annotationTarget, propertyName, metadata) {
    var metadataKey = TAGGED_PROP;
    _tagParameterOrProperty(metadataKey, annotationTarget.constructor, propertyName, metadata);
}
function _tagParameterOrProperty(metadataKey, annotationTarget, propertyName, metadata, parameterIndex) {
    var paramsOrPropertiesMetadata = {};
    var isParameterDecorator = (typeof parameterIndex === "number");
    var key = (parameterIndex !== undefined && isParameterDecorator) ? parameterIndex.toString() : propertyName;
    if (isParameterDecorator && propertyName !== undefined) {
        throw new Error(INVALID_DECORATOR_OPERATION);
    }
    if (Reflect.hasOwnMetadata(metadataKey, annotationTarget)) {
        paramsOrPropertiesMetadata = Reflect.getMetadata(metadataKey, annotationTarget);
    }
    var paramOrPropertyMetadata = paramsOrPropertiesMetadata[key];
    if (!Array.isArray(paramOrPropertyMetadata)) {
        paramOrPropertyMetadata = [];
    }
    else {
        for (var _i = 0, paramOrPropertyMetadata_1 = paramOrPropertyMetadata; _i < paramOrPropertyMetadata_1.length; _i++) {
            var m = paramOrPropertyMetadata_1[_i];
            if (m.key === metadata.key) {
                throw new Error(DUPLICATED_METADATA + " " + m.key.toString());
            }
        }
    }
    paramOrPropertyMetadata.push(metadata);
    paramsOrPropertiesMetadata[key] = paramOrPropertyMetadata;
    Reflect.defineMetadata(metadataKey, paramsOrPropertiesMetadata, annotationTarget);
}

function inject(serviceIdentifier) {
    return function (target, targetKey, index) {
        if (serviceIdentifier === undefined) {
            throw new Error(UNDEFINED_INJECT_ANNOTATION(target.name));
        }
        var metadata = new Metadata(INJECT_TAG, serviceIdentifier);
        if (typeof index === "number") {
            tagParameter(target, targetKey, index, metadata);
        }
        else {
            tagProperty(target, targetKey, metadata);
        }
    };
}

var __spreadArrays = (undefined && undefined.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};

var __spreadArrays$1 = (undefined && undefined.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};

var __awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};

var ContainerModule = (function () {
    function ContainerModule(registry) {
        this.id = id();
        this.registry = registry;
    }
    return ContainerModule;
}());

function injectable() {
    return function (target) {
        if (Reflect.hasOwnMetadata(PARAM_TYPES, target)) {
            throw new Error(DUPLICATED_INJECTABLE_DECORATOR);
        }
        var types = Reflect.getMetadata(DESIGN_PARAM_TYPES, target) || [];
        Reflect.defineMetadata(PARAM_TYPES, types, target);
        return target;
    };
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);

    if (enumerableOnly) {
      symbols = symbols.filter(function (sym) {
        return Object.getOwnPropertyDescriptor(object, sym).enumerable;
      });
    }

    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
  try {
    var info = gen[key](arg);
    var value = info.value;
  } catch (error) {
    reject(error);
    return;
  }

  if (info.done) {
    resolve(value);
  } else {
    Promise.resolve(value).then(_next, _throw);
  }
}

function _asyncToGenerator(fn) {
  return function () {
    var self = this,
        args = arguments;
    return new Promise(function (resolve, reject) {
      var gen = fn.apply(self, args);

      function _next(value) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
      }

      function _throw(err) {
        asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
      }

      _next(undefined);
    });
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

function __decorate(decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
}

function __metadata(metadataKey, metadataValue) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(metadataKey, metadataValue);
}

// use `self` instead of `window`, account for non-window contexts, such as in Web Workers
// @see https://developer.mozilla.org/en-US/docs/Web/API/Window/self
var supportsTouchEvents = ('ontouchstart' in self);
var supportsPointerEvents = !!self.PointerEvent;
function isTouchEvent(event) {
  return supportsTouchEvents && event instanceof TouchEvent;
}
function isMouseEvent(event) {
  return !self.MouseEvent || event instanceof MouseEvent && (!supportsPointerEvents || !(event instanceof self.PointerEvent));
}
/**
 * borrow from pixi/interaction
 * Ensures that the original event object contains all data that a regular pointer event would have
 */

function normalizeToPointerEvent(event) {
  if (isTouchEvent(event)) {
    // for (let i = 0; i < event.changedTouches.length; i++) {
    var touch = event.changedTouches[0]; // @ts-ignore

    return _objectSpread2({
      button: event.touches.length ? 1 : 0,
      buttons: event.touches.length ? 1 : 0,
      isPrimary: event.touches.length === 1 && event.type === 'touchstart',
      width: touch.radiusX || 1,
      height: touch.radiusY || 1,
      tiltX: 0,
      tiltY: 0,
      pointerType: 'touch',
      pointerId: touch.identifier || 0,
      pressure: touch.force || 0.5,
      twist: 0,
      tangentialPressure: 0,
      offsetX: touch.clientX,
      offsetY: touch.clientY
    }, touch); // }
  } else if (isMouseEvent(event)) {
    // @ts-ignore
    return _objectSpread2({
      isPrimary: true,
      width: 1,
      height: 1,
      tiltX: 0,
      tiltY: 0,
      pointerType: 'mouse',
      pointerId: 10,
      pressure: 0.5,
      twist: 0,
      tangentialPressure: 0
    }, event);
  } else {
    return event;
  }
}

var _class, _temp;

var DOMInteractionPlugin_1;
/**
 * listen to mouse/touch/pointer events on DOM wrapper, trigger pointer events
 */

var DOMInteractionPlugin = DOMInteractionPlugin_1 = (_temp = _class = /*#__PURE__*/function () {
  function DOMInteractionPlugin() {
    _classCallCheck(this, DOMInteractionPlugin);

    this.contextService = void 0;
  }

  _createClass(DOMInteractionPlugin, [{
    key: "apply",
    value: function apply(renderingService) {
      var _this = this;

      var onPointerMove = function onPointerMove(ev) {
        renderingService.hooks.pointerMove.call(normalizeToPointerEvent(ev), ev);
      };

      var onPointerUp = function onPointerUp(ev) {
        renderingService.hooks.pointerUp.call(normalizeToPointerEvent(ev), ev);
      };

      var onPointerDown = function onPointerDown(ev) {
        renderingService.hooks.pointerDown.call(normalizeToPointerEvent(ev), ev);
      };

      var onPointerCancel = function onPointerCancel(ev) {
        renderingService.hooks.pointerCancel.call(normalizeToPointerEvent(ev), ev);
      };

      var onPointerOver = function onPointerOver(ev) {
        renderingService.hooks.pointerOver.call(normalizeToPointerEvent(ev), ev);
      };

      var onPointerOut = function onPointerOut(ev) {
        renderingService.hooks.pointerOut.call(normalizeToPointerEvent(ev), ev);
      };

      var onPointerWheel = function onPointerWheel(ev) {
        renderingService.hooks.pointerWheel.call(normalizeToPointerEvent(ev), ev);
      };

      renderingService.hooks.init.tap(DOMInteractionPlugin_1.tag, function () {
        var $el = _this.contextService.getDomElement();

        if (supportsPointerEvents) {
          self.document.addEventListener('pointermove', onPointerMove, true);
          $el.addEventListener('pointerdown', onPointerDown, true); // pointerout is fired in addition to pointerup (for touch events) and pointercancel
          // we already handle those, so for the purposes of what we do in onPointerOut, we only
          // care about the pointerleave event

          $el.addEventListener('pointerleave', onPointerOut, true);
          $el.addEventListener('pointerover', onPointerOver, true);
          self.addEventListener('pointercancel', onPointerCancel, true);
          self.addEventListener('pointerup', onPointerUp, true);
          $el.addEventListener('wheel', onPointerWheel, true);
        } else {
          self.document.addEventListener('mousemove', onPointerMove, true);
          $el.addEventListener('mousedown', onPointerDown, true);
          $el.addEventListener('mouseout', onPointerOut, true);
          $el.addEventListener('mouseover', onPointerOver, true);
          self.addEventListener('mouseup', onPointerUp, true);
          $el.addEventListener('wheel', onPointerWheel, true);
        } // always look directly for touch events so that we can provide original data
        // In a future version we should change this to being just a fallback and rely solely on
        // PointerEvents whenever available


        if (supportsTouchEvents) {
          $el.addEventListener('touchstart', onPointerDown, true);
          $el.addEventListener('touchcancel', onPointerCancel, true);
          $el.addEventListener('touchend', onPointerUp, true);
          $el.addEventListener('touchmove', onPointerMove, true);
        }
      });
      renderingService.hooks.destroy.tap(DOMInteractionPlugin_1.tag, /*#__PURE__*/_asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
        var $el;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                $el = _this.contextService.getDomElement();

                if (supportsPointerEvents) {
                  self.document.removeEventListener('pointermove', onPointerMove, true);
                  $el.removeEventListener('pointerdown', onPointerDown, true);
                  $el.removeEventListener('pointerleave', onPointerOut, true);
                  $el.removeEventListener('pointerover', onPointerOver, true);
                  self.removeEventListener('pointercancel', onPointerCancel, true);
                  self.removeEventListener('pointerup', onPointerUp, true);
                } else {
                  self.document.removeEventListener('mousemove', onPointerMove, true);
                  $el.removeEventListener('mousedown', onPointerDown, true);
                  $el.removeEventListener('mouseout', onPointerOut, true);
                  $el.removeEventListener('mouseover', onPointerOver, true);
                  self.removeEventListener('mouseup', onPointerUp, true);
                }

                if (supportsTouchEvents) {
                  $el.removeEventListener('touchstart', onPointerDown, true);
                  $el.removeEventListener('touchcancel', onPointerCancel, true);
                  $el.removeEventListener('touchend', onPointerUp, true);
                  $el.removeEventListener('touchmove', onPointerMove, true);
                }

              case 3:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      })));
    }
  }]);

  return DOMInteractionPlugin;
}(), _class.tag = 'DOMInteractionPlugin', _temp);

__decorate([inject(ContextService), __metadata("design:type", Object)], DOMInteractionPlugin.prototype, "contextService", void 0);

DOMInteractionPlugin = DOMInteractionPlugin_1 = __decorate([injectable()], DOMInteractionPlugin);

var containerModule = new ContainerModule(function (bind, unbind, isBound, rebind) {
  bind(DOMInteractionPlugin).toSelf().inSingletonScope();
  bind(RenderingPluginContribution).toService(DOMInteractionPlugin);
});

export { containerModule };
